@@ -97,16 +97,19 @@ class Transcription(models.Model):
                 BPprob += float(sum([x.get_frequency()
                                     for x in count])) / float(sum([x.get_frequency()
                                                                     for x in totCount]))
-        SPprob = SPprob / float(len(phones))
-        BPprob = BPprob / float(len(phones)-1)
+        try:
+            SPprob = SPprob / float(len(phones))
+        except ZeroDivisionError:
+            SPprob = 0.0
+        try:
+            BPprob = BPprob / float(len(phones)-1)
+        except ZeroDivisionError:
+            BPprob = 0.0
         self.sphone_prob,self.biphone_prob = SPprob,BPprob
         self.save()
         return SPprob,BPprob
 
-
-    def get_neigh_densities(self):
-        if self.neigh_den is not None and self.fwnd is not None:
-            return self.neigh_den,self.fwnd
+    def get_neighbours(self):
         any_segment = '.'
         phones = self.strip_transcription()
         patterns = []
@@ -122,10 +125,17 @@ class Transcription(models.Model):
             patt += any_segment
             patt += phones[i:]
             patterns.append('^'+''.join(patt) +'$')
-
-        neighs = Transcription.objects.extra(
+        patt = phones + [any_segment]
+        patterns.append('^'+''.join(patt) +'$')
+        neighs = Transcription.objects.exclude(pk=self.pk).extra(
                     where = [UR_LOOKUP],
                     params = ['|'.join(patterns)])
+        return neighs
+
+    def get_neigh_densities(self):
+        if self.neigh_den is not None and self.fwnd is not None:
+            return self.neigh_den,self.fwnd
+        neighs = self.get_neighbours()
         nd = len(neighs)
         fwnd = sum([ x.get_frequency() for x in neighs])
         self.neigh_den = nd
@@ -198,8 +208,11 @@ class WordForm(models.Model):
             nd,fwnd = t.get_neigh_densities()
             nd_sum += nd
             fwnd_sum += fwnd
-        nd = float(nd_sum)/float(len(self.transcriptions.all()))
-        fwnd = float(fwnd_sum)/float(len(self.transcriptions.all()))
+        try:
+            nd = float(nd_sum)/float(len(self.transcriptions.all()))
+            fwnd = float(fwnd_sum)/float(len(self.transcriptions.all()))
+        except ZeroDivisionError:
+            nd, fwnd = nd_sum,fwnd_sum
         return nd,fwnd
 
     def get_phono_prob(self):
@@ -209,8 +222,11 @@ class WordForm(models.Model):
             sp,bp = t.get_phono_prob()
             sp_sum += sp
             bp_sum += bp
-        sp = float(sp_sum)/float(len(self.transcriptions.all()))
-        bp = float(bp_sum)/float(len(self.transcriptions.all()))
+        try:
+            sp = float(sp_sum)/float(len(self.transcriptions.all()))
+            bp = float(bp_sum)/float(len(self.transcriptions.all()))
+        except ZeroDivisionError:
+            sp,bp = sp_sum,bp_sum
         return sp,bp
 
 
